@@ -37,9 +37,10 @@
 
 #include "cspublic.h"
 #include "ctlib.h"
-#include <freetds/convert.h>
 #include "replacements.h"
 
+#undef cs_dt_crack
+
 static CS_INT cs_diag_storemsg(CS_CONTEXT *context, CS_CLIENTMSG *message);
 static CS_INT cs_diag_clearmsg(CS_CONTEXT *context, CS_INT type);
 static CS_INT cs_diag_getmsg(CS_CONTEXT *context, CS_INT idx, CS_CLIENTMSG *message);
@@ -330,6 +331,10 @@ cs_ctx_alloc(CS_INT version, CS_CONTEXT ** ctx)
 		/* set default in case there's no locale file */
 		tds_ctx->locale->date_fmt = strdup(STD_DATETIME_FMT);
 	}
+
+	(*ctx)->login_timeout = -1;
+	(*ctx)->query_timeout = -1;
+
 	return CS_SUCCEED;
 }
 
@@ -832,37 +837,71 @@ cs_convert(CS_CONTEXT * ctx, CS_DATAFMT * srcfmt, CS_VOID * srcdata, CS_DATAFMT
 }
 
 CS_RETCODE
-cs_dt_crack(CS_CONTEXT * ctx, CS_INT datetype, CS_VOID * dateval, CS_DATEREC * daterec)
+cs_dt_crack_v2(CS_CONTEXT * ctx, CS_INT datetype, CS_VOID * dateval, CS_DATEREC * daterec)
 {
-	TDS_DATETIME *dt;
-	TDS_DATETIME4 *dt4;
 	TDSDATEREC dr;
+	TDS_INT tds_type;
+	bool extended = false;
 
-	tdsdump_log(TDS_DBG_FUNC, "cs_dt_crack(%p, %d, %p, %p)\n", ctx, datetype, dateval, daterec);
+	tdsdump_log(TDS_DBG_FUNC, "cs_dt_crack_v2(%p, %d, %p, %p)\n", ctx, datetype, dateval, daterec);
 
-	if (datetype == CS_DATETIME_TYPE) {
-		dt = (TDS_DATETIME *) dateval;
-		tds_datecrack(SYBDATETIME, dt, &dr);
-	} else if (datetype == CS_DATETIME4_TYPE) {
-		dt4 = (TDS_DATETIME4 *) dateval;
-		tds_datecrack(SYBDATETIME4, dt4, &dr);
-	} else {
+	switch (datetype) {
+	case CS_DATETIME_TYPE:
+		tds_type = SYBDATETIME;
+		break;
+	case CS_DATETIME4_TYPE:
+		tds_type = SYBDATETIME4;
+		break;
+	case CS_DATE_TYPE:
+		tds_type = SYBDATE;
+		break;
+	case CS_TIME_TYPE:
+		tds_type = SYBTIME;
+		break;
+	case CS_BIGDATETIME_TYPE:
+		tds_type = SYB5BIGDATETIME;
+		extended = true;
+		break;
+	case CS_BIGTIME_TYPE:
+		tds_type = SYB5BIGTIME;
+		extended = true;
+		break;
+	default:
 		return CS_FAIL;
 	}
-	daterec->dateyear = dr.year;
-	daterec->datemonth = dr.month;
-	daterec->datedmonth = dr.day;
-	daterec->datedyear = dr.dayofyear;
-	daterec->datedweek = dr.weekday;
+	tds_datecrack(tds_type, dateval, &dr);
+
+	/* Sybase CT-Library does not set these fields for CS_BIGTIME_TYPE */
+	if (tds_type != SYB5BIGTIME) {
+		daterec->dateyear = dr.year;
+		daterec->datemonth = dr.month;
+		daterec->datedmonth = dr.day;
+		daterec->datedyear = dr.dayofyear;
+		daterec->datedweek = dr.weekday;
+	}
 	daterec->datehour = dr.hour;
 	daterec->dateminute = dr.minute;
 	daterec->datesecond = dr.second;
 	daterec->datemsecond = dr.decimicrosecond / 10000u;
 	daterec->datetzone = 0;
+	if (extended) {
+		daterec->datesecfrac = dr.decimicrosecond / 10u;
+		daterec->datesecprec = 1000000;
+	}
 
 	return CS_SUCCEED;
 }
 
+CS_RETCODE
+cs_dt_crack(CS_CONTEXT * ctx, CS_INT datetype, CS_VOID * dateval, CS_DATEREC * daterec)
+{
+	tdsdump_log(TDS_DBG_FUNC, "cs_dt_crack(%p, %d, %p, %p)\n", ctx, datetype, dateval, daterec);
+
+	if (datetype != CS_BIGDATETIME_TYPE && datetype != CS_BIGTIME_TYPE)
+		return cs_dt_crack_v2(ctx, datetype, dateval, daterec);
+	return CS_FAIL;
+}
+
 CS_RETCODE
 cs_loc_alloc(CS_CONTEXT * ctx, CS_LOCALE ** locptr)
 {
@@ -1254,6 +1293,8 @@ cs_will_convert(CS_CONTEXT * ctx, CS_INT srctype, CS_INT desttype, CS_BOOL * res
 
 	tdsdump_log(TDS_DBG_FUNC, "cs_will_convert(%p, %d, %d, %p)\n", ctx, srctype, desttype, result);
 
+	srctype = _ct_get_server_type(NULL, srctype);
+	desttype = _ct_get_server_type(NULL, desttype);
 	*result = (tds_willconvert(srctype, desttype) ? CS_TRUE : CS_FALSE);
 	return CS_SUCCEED;
 }
@@ -1365,3 +1406,43 @@ cs_diag_countmsg(CS_CONTEXT *context, CS_INT *count)
 	*count = msg_count;
 	return CS_SUCCEED;
 }
+
+/**
+ * Try to convert to a type we can handle
+ */
+int
+_cs_convert_not_client(CS_CONTEXT *ctx, const TDSCOLUMN *curcol, CONV_RESULT *convert_buffer, unsigned char **p_src)
+{
+	int ct_type;
+	TDS_SERVER_TYPE desttype, type = curcol->column_type;
+
+	if (type == SYBVARIANT)
+		type = ((const TDSVARIANT *) curcol->column_data)->type;
+
+	switch (type) {
+	case SYBMSDATE:
+		desttype = SYBDATE;
+		ct_type = CS_DATE_TYPE;
+		break;
+	case SYBMSTIME:
+		desttype = SYB5BIGTIME;
+		ct_type = CS_BIGTIME_TYPE;
+		break;
+	case SYBMSDATETIME2:
+	case SYBMSDATETIMEOFFSET:
+		desttype = SYB5BIGDATETIME;
+		ct_type = CS_BIGDATETIME_TYPE;
+		break;
+	default:
+		return CS_ILLEGAL_TYPE;
+	}
+
+	if (convert_buffer) {
+		int len = tds_convert(ctx->tds_ctx, type, (TDS_CHAR *) *p_src,
+				      curcol->column_cur_size, desttype, convert_buffer);
+		if (len < 0)
+			return CS_ILLEGAL_TYPE; // TODO _csclient_msg ??
+		*p_src = (unsigned char *) convert_buffer;
+	}
+	return ct_type;
+}
