@@ -38,6 +38,7 @@
 
 #include <freetds/tds.h>
 #include <freetds/iconv.h>
+#include <freetds/bool.h>
 #if HAVE_ICONV
 #include <iconv.h>
 #endif
@@ -324,6 +325,10 @@ tds_iconv_open(TDSCONNECTION * conn, const char *charset, int use_utf16)
 
 	tdsdump_log(TDS_DBG_FUNC, "tds_iconv_open(%p, %s)\n", conn, charset);
 
+	/* TDS 5.0 support only UTF-16 encodings */
+	if (IS_TDS50(conn))
+		use_utf16 = true;
+
 	/* initialize */
 	if (!iconv_initialized) {
 		if ((ret = tds_iconv_init()) > 0) {
@@ -553,11 +558,11 @@ tds_iconv(TDSSOCKET * tds, TDSICONV * conv, TDS_ICONV_DIRECTION io,
 	iconv_t error_cd = invalid;
 
 	char quest_mark[] = "?";	/* best to leave non-const; implementations vary */
-	ICONV_CONST char *pquest_mark = quest_mark;
+	ICONV_CONST char *pquest_mark;
 	size_t lquest_mark;
 	size_t irreversible;
 	size_t one_character;
-	int eilseq_raised = 0;
+	bool eilseq_raised = false;
 	int conv_errno;
 	/* cast away const-ness */
 	TDS_ERRNO_MESSAGE_FLAGS *suppress = (TDS_ERRNO_MESSAGE_FLAGS*) &conv->suppress;
@@ -592,12 +597,13 @@ tds_iconv(TDSSOCKET * tds, TDSICONV * conv, TDS_ICONV_DIRECTION io,
 		size_t len = *inbytesleft < *outbytesleft ? *inbytesleft : *outbytesleft;
 
 		memcpy(*outbuf, *inbuf, len);
-		errno = *inbytesleft > *outbytesleft ? E2BIG : 0;
+		conv_errno = *inbytesleft > *outbytesleft ? E2BIG : 0;
 		*inbytesleft -= len;
 		*outbytesleft -= len;
 		*inbuf += len;
 		*outbuf += len;
-		return 0;
+		errno = conv_errno;
+		return conv_errno ? (size_t) -1 : 0;
 	}
 
 	/*
@@ -625,7 +631,7 @@ tds_iconv(TDSSOCKET * tds, TDSICONV * conv, TDS_ICONV_DIRECTION io,
 		conv_errno = errno;
 
 		if (conv_errno == EILSEQ)
-			eilseq_raised = 1;
+			eilseq_raised = true;
 
 		if (conv_errno != EILSEQ || io != to_client || !inbuf)
 			break;
@@ -707,7 +713,7 @@ tds_iconv(TDSSOCKET * tds, TDSICONV * conv, TDS_ICONV_DIRECTION io,
 /**
  * Get a iconv info structure, allocate and initialize if needed
  */
-static TDSICONV *
+TDSICONV *
 tds_iconv_get_info(TDSCONNECTION * conn, int canonic_client, int canonic_server)
 {
 	TDSICONV *info;
@@ -1028,7 +1034,7 @@ collate2charset(int sql_collate, int lcid)
 	case 0x41c:
 	case 0x424:
 	case 0x442:
-		/* case 0x81a: seem wrong in XP table TODO check */
+	case 0x81a:
 	case 0x104e:		/* ?? */
 	case 0x141a:
 		cp = TDS_CHARSET_CP1250;
@@ -1042,10 +1048,12 @@ collate2charset(int sql_collate, int lcid)
 	case 0x440:
 	case 0x444:
 	case 0x450:
-	case 0x81a:		/* ?? */
 	case 0x82c:
 	case 0x843:
 	case 0xc1a:
+	case 0x46d:
+	case 0x201a:
+	case 0x485:
 		cp = TDS_CHARSET_CP1251;
 		break;
 	case 0x1007:
@@ -1061,7 +1069,6 @@ collate2charset(int sql_collate, int lcid)
 	case 0x180c:
 	case 0x1c09:
 	case 0x1c0a:
-	case 0x201a:
 	case 0x2009:
 	case 0x200a:
 	case 0x2409:
@@ -1109,8 +1116,6 @@ collate2charset(int sql_collate, int lcid)
 	case 0x440a:
 	case 0x441:
 	case 0x456:
-	case 0x46d:
-	case 0x485:
 	case 0x480a:
 	case 0x4c0a:
 	case 0x500a:
